% function [f,J,] = dXdT(t,x,T,BX,K_BX,par)
%
%
%
% Output parameters:
%   f     time derivatives of the model
%   J     flux
%
%
% Mandatory input parameters:
%   t     time
%   x     state variables at t=0
%   T     temperature in degreesCelcius
%   BX    buffer sizes
%   K_BX  proton buffer dissociation constants ( cytoplasm )
%   par   parameter vector for the free parameters
%   
%
% State Variables:
% [ketoglutarate_cytoplasm , aspartate_cytoplasm , oxaloacetate_cytoplasm , glutamate_cytoplasm ]
%
%
% Free Parameters:
% [Vmf_GOT ]

function [f,J,varargout] = dXdT(t,x,T,BX,K_BX,par)
%% GLOBAL VARIABLES
% minimal parameter value
    MinCon = 0;


%% LIST OF STATE VARIABLES
% 1 ketoglutarate_cytoplasm
% 2 aspartate_cytoplasm
% 3 oxaloacetate_cytoplasm
% 4 glutamate_cytoplasm
% 5 h_cytoplasm
% 6 m_cytoplasm
% 7 k_cytoplasm

% PARTIAL VOLUME FRACTIONS
VWater_cytoplasm = 1; % [=] l water (l region)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 
VRegion_cytoplasm = 1; % [=] l region (l tissue)^{-1}, Vinnakota and Bassingthwaighte, AJP, 2004 

%% THERMODYNAMIC DATA
RT = 8.314*(T+273.15)/1e3; % kJ  mol^{-1}
F = 0.096484; % kJ mol^{-1} mV^{-1}

%% STATE VARIABLES

% Concentrations of Reference Species
ketoglutarate_cytoplasm = x(1);
aspartate_cytoplasm = x(2);
oxaloacetate_cytoplasm = x(3);
glutamate_cytoplasm = x(4);
% Concentrations of H, Mg, and K
h_cytoplasm = x(5);
m_cytoplasm = x(6);
k_cytoplasm = x(7);

% Membrane potentials

%% DISSOCIATION CONSTANTS
% ketoglutarate_cytoplasm
Kh(1) = Inf;
Km(1) = Inf;
Kk(1) = Inf;
% aspartate_cytoplasm
Kh(2) = 0.00021342954599320779120880109402946800400968641042709;
Km(2) = Inf;
Kk(2) = Inf;
% oxaloacetate_cytoplasm
Kh(3) = 0.00015083741503259175554402249375129940744955092668533;
Km(3) = 0.13709390976603802458910763562016654759645462036133;
Kk(3) = Inf;
% glutamate_cytoplasm
Kh(4) = 7.7491584389679694184691061131786682381061837077141e-05;
Km(4) = Inf;
Kk(4) = Inf;

%% BINDING POLYNOMIALS
P( 1 ) = 1  + h_cytoplasm/Kh(1) + m_cytoplasm/Km(1) + k_cytoplasm/Kk(1);
P( 2 ) = 1  + h_cytoplasm/Kh(2) + m_cytoplasm/Km(2) + k_cytoplasm/Kk(2);
P( 3 ) = 1  + h_cytoplasm/Kh(3) + m_cytoplasm/Km(3) + k_cytoplasm/Kk(3);
P( 4 ) = 1  + h_cytoplasm/Kh(4) + m_cytoplasm/Km(4) + k_cytoplasm/Kk(4);

%% THERMODYNAMIC EQUATIONS
DGro_AAT =-1.47;

Keq_AAT_cytoplasm = exp(-DGro_AAT/RT)/P(1)/P(2)*P(3)*P(4);

%% FLUX EQUATIONS
%AAT_cytoplasm
a=aspartate_cytoplasm;
b=ketoglutarate_cytoplasm;
p=oxaloacetate_cytoplasm;
q=glutamate_cytoplasm;
KmA=3900e-6;
KmB=430e-6;
KmP=88e-6;
KmQ=8900e-6;
Kia=3480e-6;
Kib=710e-6;
Kip=50e-6;
Kiq=8400e-6;
KiAKG=16.6e-3;
ai=1+b/KiAKG;
Vmf_GOT=par(1);
Vmr=Vmf_GOT/Keq_AAT_cytoplasm*(KmQ*Kip/Kia/KmB);
if(a>MinCon)&&(b>MinCon)
ab=a*b;
else
ab=0;
end
if(p>MinCon)&&(q>MinCon)
pq=p*q;
else
pq=0;
end
J_AAT_cytoplasm=Vmf_GOT*Vmr*(ab-pq/Keq_AAT_cytoplasm)/(Vmr*KmB*a+Vmr*KmA*ai*b+Vmf_GOT*KmQ/Keq_AAT_cytoplasm*ai*p+Vmf_GOT*KmP/Keq_AAT_cytoplasm*q+Vmr*a*b+Vmf_GOT*KmQ/Kia/Keq_AAT_cytoplasm*a*p+Vmf_GOT/Keq_AAT_cytoplasm*p*q+Vmr*KmA/Kiq*b*q);

%% REACTANT TIME DERIVATIVES
f(1,:) = ( 0  - 1*J_AAT_cytoplasm ) / VWater_cytoplasm; % ketoglutarate_cytoplasm
f(2,:) = ( 0  - 1*J_AAT_cytoplasm ) / VWater_cytoplasm; % aspartate_cytoplasm
f(3,:) = ( 0  + 1*J_AAT_cytoplasm ) / VWater_cytoplasm; % oxaloacetate_cytoplasm
f(4,:) = ( 0  + 1*J_AAT_cytoplasm ) / VWater_cytoplasm; % glutamate_cytoplasm

%% ION EQUATIONS
% COMPARTMENT cytoplasm:
ii = [1  2  3  4]; % Indices of SVs in compartment cytoplasm
% PARTIAL DERIVATIVES
pHBpM = -sum( (h_cytoplasm*x(ii)'./Kh(ii))./(Km(ii).*P(ii).^2) );
pHBpK = -sum( (h_cytoplasm*x(ii)'./Kh(ii))./(Kk(ii).*P(ii).^2) );
pHBpH = +sum( (1+m_cytoplasm./Km(ii)+k_cytoplasm./Kk(ii)).*x(ii)'./(Kh(ii).*P(ii).^2) );
pMBpH = -sum( (m_cytoplasm*x(ii)'./Km(ii))./(Kh(ii).*P(ii).^2) );
pMBpK = -sum( (m_cytoplasm*x(ii)'./Km(ii))./(Kk(ii).*P(ii).^2) );
pMBpM = +sum( (1+h_cytoplasm./Kh(ii)+k_cytoplasm./Kk(ii)).*x(ii)'./(Km(ii).*P(ii).^2) );
pKBpH = -sum( (k_cytoplasm*x(ii)'./Kk(ii))./(Kh(ii).*P(ii).^2) );
pKBpM = -sum( (k_cytoplasm*x(ii)'./Kk(ii))./(Km(ii).*P(ii).^2) );
pKBpK = +sum( (1+h_cytoplasm./Kh(ii)+m_cytoplasm./Km(ii)).*x(ii)'./(Kk(ii).*P(ii).^2) );
% PHIs
J_H = (0 + 0*J_AAT_cytoplasm) / VWater_cytoplasm;
J_M = (0) / VWater_cytoplasm;
J_K = (0) / VWater_cytoplasm;
Phi_H = J_H - sum( h_cytoplasm*f(ii)'./(Kh(ii).*P(ii)) );
Phi_M = -sum( m_cytoplasm*f(ii)'./(Km(ii).*P(ii)) );
Phi_K = J_K -sum( k_cytoplasm*f(ii)'./(Kk(ii).*P(ii)) );
% ALPHAs
aH = 1 + pHBpH;
aM = 1 + pMBpM;
aK = 1 + pKBpK;
% ADDITIONAL BUFFER for [H+]
aH = 1 + pHBpH + BX(1)/K_BX(1)/(1+h_cytoplasm/K_BX(1))^2; % M
% DENOMINATOR
D = aH*pKBpM*pMBpK + aK*pHBpM*pMBpH + aM*pHBpK*pKBpH - ...
    aM*aK*aH - pHBpK*pKBpM*pMBpH - pHBpM*pMBpK*pKBpH;
% DERIVATIVES for H,Mg,K
f(5,:) =  ( (pKBpM*pMBpK - aM*aK)*Phi_H + ...
            (aK*pHBpM - pHBpK*pKBpM)*Phi_M + ...
            (aM*pHBpK - pHBpM*pMBpK)*Phi_K ) / D;
f(6,:) =  ( (aK*pMBpH - pKBpH*pMBpK)*Phi_H + ...
            (pKBpH*pHBpK - aH*aK)*Phi_M + ...
            (aH*pMBpK - pHBpK*pMBpH)*Phi_K ) / D;
f(7,:) =  ( (aM*pKBpH - pKBpM*pMBpH)*Phi_H + ...
            (aH*pKBpM - pKBpH*pHBpM)*Phi_M + ...
            (pMBpH*pHBpM - aH*aM)*Phi_K ) / D;

%% ELECTROPHYS EQUATIONS
%% FLUX VECTOR:
J = [ J_AAT_cytoplasm];
